"""Pressure Profile from file"""
import typing as t

import numpy as np

from taurex.types import PathLike
from taurex.util import conversion_factor

from .arraypressure import ArrayPressureProfile


class FilePressureProfile(ArrayPressureProfile):
    """Load pressure profile from file."""

    def __init__(
        self,
        filename: t.Optional[PathLike] = None,
        usecols: t.Optional[int] = 0,
        skiprows: t.Optional[int] = 0,
        units: t.Optional[str] = "Pa",
        delimiter: t.Optional[str] = None,
        reverse: t.Optional[bool] = False,
        is_central: t.Optional[bool] = True,
    ):
        """Initialize the pressure profile.

        Parameters
        ----------
        filename : t.Optional[PathLike], optional
            Filename to load, by default None
        usecols : t.Optional[int], optional
            Which column to use, by default 0
        skiprows : t.Optional[int], optional
            How many rows to skip, by default 0
        units : t.Optional[str], optional
            Units of the pressure, by default "Pa"
        delimiter : t.Optional[str], optional
            Delimiter to use, by default None
        reverse : t.Optional[bool], optional
            Reverse the pressure profile, by default False
        is_central : t.Optional[bool], optional
            If the pressure profile is central or not, by default True
            **New in v3.2.0**


        """
        to_pa = conversion_factor(units, "Pa")

        read_arr = np.loadtxt(
            filename,
            usecols=int(usecols),
            skiprows=int(skiprows),
            delimiter=delimiter,
            dtype=np.float64,
        )
        super().__init__(read_arr * to_pa, reverse=reverse, is_central=is_central)

    @classmethod
    def input_keywords(cls) -> t.Tuple[str, ...]:
        return (
            "file",
            "fromfile",
            "loadfile",
        )



"""
Atmospheric chemistry related modules
"""

from .taurexchemistry import TaurexChemistry
from .gas.constantgas import ConstantGas
from .gas.twolayergas import TwoLayerGas
from .gas.powergas import PowerGas
from .gas.customgas import CustomGas
from .gas.twopointgas import TwoPointGas
from .filechemistry import ChemistryFile
from .autochemistry import AutoChemistry


"""Main logging module."""

import logging

from .logger import Loggable
from .logger import Loggable as Logger
from .logger import setup_log

last_log = logging.INFO


def setLogLevel(level: int) -> None:  # noqa: N802
    """Set the log level.

    Parameters
    ----------
    level : int
        Log level to set.

    """
    global last_log
    from .logger import root_logger

    root_logger.setLevel(level)
    last_log = level


def disableLogging() -> None:  # noqa: N802
    """Disable logging."""
    import logging

    from .logger import root_logger

    global last_log
    last_log = root_logger.level
    root_logger.setLevel(logging.CRITICAL)


def enableLogging() -> None:  # noqa: N802
    """Enable logging."""
    global last_log
    import logging

    if last_log is None:
        last_log = logging.INFO
    setLogLevel(last_log)


__all__ = [
    "Logger",
    "Loggable",
    "setLogLevel",
    "disableLogging",
    "enableLogging",
    "setup_log",
]



"""Types for TauREx 3"""
import os
import pathlib
import typing as t

import numpy as np
import numpy.typing as npt

T = t.TypeVar("T")

ModelOutputType = t.Tuple[
    npt.NDArray[np.float64],
    npt.NDArray[np.float64],
    t.Union[npt.NDArray[np.float64], None],
    t.Union[t.Dict, T, None],
]
"""Model output type."""

ScalarType = t.Union[float, int, np.float64, np.int64]
"""Scalar type."""

ArrayType = t.Union[np.ndarray, npt.ArrayLike]
"""Array type."""

AnyValType = t.Union[ScalarType, ArrayType]
"""Any value type."""

PathLike = t.Union[str, bytes, os.PathLike, pathlib.Path]
"""Path like type."""
